/***************************************************************************
                          configdialog.cpp  -  description
                             -------------------
    begin                : Fri Jun 25 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kdialogbase.h>
#include <kfiledialog.h>
#include <klistview.h>

#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qlineedit.h>

#include "serversettingsdialog.h"
#include "server.h"
#include "configdialog.h"


ConfigDialog::ConfigDialog(QWidget* parent) : KDialogBase(
	4,i18n("Knzb configuration"), Ok|Apply|Cancel, Ok,parent,NULL,true) {

	QSize initialSize(650,450);
	setInitialSize(initialSize);

	//General page
	KIconLoader l("knzb");
	QWidget *generalPage = addPage(i18n("General"),i18n("General settings"),l.loadIcon("configure.png",KIcon::Toolbar));
	QVBoxLayout *generalLayout = new QVBoxLayout(generalPage);
	QGroupBox *pathsGroupBox = new QGroupBox(3,Qt::Horizontal,"Paths",generalPage);
	generalLayout->addWidget(pathsGroupBox);

	new QLabel(i18n("Temporary file path"),pathsGroupBox);
	m_tmpPathLineEdit = new QLineEdit(pathsGroupBox);
	m_tmpPathLineEdit->setReadOnly(true);
	QToolTip::add(m_tmpPathLineEdit,i18n("The folder used to store temporary files (downloaded posts)"));
	m_tmpPathBrowseBtn = new QPushButton("Browse", pathsGroupBox);

	new QLabel(i18n("Destination file path"),pathsGroupBox);
	m_destPathLineEdit = new QLineEdit(pathsGroupBox);
	m_destPathLineEdit->setReadOnly(true);
	QToolTip::add(m_destPathLineEdit,i18n("The folder used to store completed files"));
	m_destPathBrowseBtn = new QPushButton("Browse",pathsGroupBox);


	connect(m_tmpPathBrowseBtn, SIGNAL(clicked()), this, SLOT(tmpPathBrowseClicked()));
	connect(m_destPathBrowseBtn, SIGNAL(clicked()), this, SLOT(destPathBrowseClicked()));
	
	QGroupBox *downloadGroupBox = new QGroupBox(5,Qt::Vertical,"Downloads",generalPage);
	generalLayout->addWidget(downloadGroupBox);
	
	m_skipDownloadedFiles = new QCheckBox("Skip files if already in download directory",downloadGroupBox);
	QToolTip::add(m_skipDownloadedFiles,
			i18n("Whether a file should be skipped if a file with the same name already exists in the destination directory"));
	
	m_removeCompletedItemsFromQueue = new QCheckBox("Remove completed items from queue",downloadGroupBox);
	QToolTip::add(m_removeCompletedItemsFromQueue,i18n("Whether completed files should be removed from the queue display"));
	
	m_closeAppWhenComplete = new QCheckBox("Close application when queue is completed",downloadGroupBox);
	QToolTip::add(m_closeAppWhenComplete,i18n("Whether the application should close when the queue becomes empty"));
	
	m_dockToSystray = new QCheckBox("Dock to system tray",downloadGroupBox);
	QToolTip::add(m_dockToSystray,i18n("Whether the application should dock to the system tray when closed"));

	m_treePerNzb = new QCheckBox("Show each NZB file in a separate tree hierarchy", downloadGroupBox);
	QToolTip::add(m_dockToSystray,i18n("Whether each NZB file should be shown in a subtree in the main treeview"));

	//Server page
	QWidget *serverPage = addPage(i18n("Server"),i18n("News server settings"),l.loadIcon("connect_creating.png",KIcon::Toolbar));
	QHBoxLayout *serverLayout = new QHBoxLayout(serverPage);
	QVBoxLayout *ly1 = new QVBoxLayout(serverLayout);
	m_serverListView = new KListView(serverPage);
	m_serverListView->addColumn("Server name");
	m_serverListView->addColumn("Priority");
	m_serverListView->addColumn("Max threads");
	connect(m_serverListView,SIGNAL(selectionChanged()),this,SLOT(serverSelectionChanged()));
	
	ly1->addWidget(m_serverListView);

	QHBoxLayout *ly2 = new QHBoxLayout(ly1);
	m_addServerButton = new QPushButton(i18n("Add"),serverPage);
	QToolTip::add(m_addServerButton,i18n("Add a new server"));
	m_modifyServerButton = new QPushButton(i18n("Modify"),serverPage);
	QToolTip::add(m_modifyServerButton,i18n("Modify the selected server"));
	m_removeServerButton = new QPushButton(i18n("Remove"),serverPage);
	QToolTip::add(m_removeServerButton,i18n("Remove the selected server"));
	
	ly2->addWidget(m_addServerButton);
	ly2->addWidget(m_modifyServerButton);
	ly2->addWidget(m_removeServerButton);

	m_modifyServerButton->setEnabled(false);
	m_removeServerButton->setEnabled(false);

	connect(m_addServerButton,SIGNAL(clicked()),this,SLOT(addServerClicked()));
	connect(m_modifyServerButton,SIGNAL(clicked()),this,SLOT(modifyServerClicked()));
	connect(m_removeServerButton,SIGNAL(clicked()),this,SLOT(deleteServerClicked()));
	parseConfig();
}

ConfigDialog::~ConfigDialog(){
}

void ConfigDialog::parseConfig() {
	m_settings.loadSettings();
	
	m_tmpPathLineEdit->setText(m_settings.tempDir);
	m_destPathLineEdit->setText(m_settings.destDir);

	m_skipDownloadedFiles->setChecked(m_settings.skipDownloadedFiles);
	m_removeCompletedItemsFromQueue->setChecked(m_settings.removeCompletedItemsFromQueue);
	m_closeAppWhenComplete->setChecked(m_settings.closeAppWhenComplete);
	m_dockToSystray->setChecked(m_settings.dockToSystray);
	m_treePerNzb->setChecked(m_settings.treePerNzb);
	
	redrawServerList();
}

void ConfigDialog::exportConfig() {
	m_settings.tempDir = m_tmpPathLineEdit->text();
	m_settings.destDir = m_destPathLineEdit->text();

	m_settings.skipDownloadedFiles = m_skipDownloadedFiles->isChecked();
	m_settings.removeCompletedItemsFromQueue = m_removeCompletedItemsFromQueue->isChecked();
	m_settings.closeAppWhenComplete = m_closeAppWhenComplete->isChecked();
	m_settings.dockToSystray = m_dockToSystray->isChecked();
	m_settings.treePerNzb = m_treePerNzb->isChecked();

	//Servers are already stored.
	m_settings.saveSettings();
}

void ConfigDialog::slotOk() {
	//Make sure config is exported before the signal is emitted
	exportConfig();
	KDialogBase::slotOk();
}

void ConfigDialog::slotApply() {
	//Make sure config is exported before the signal is emitted
	exportConfig();
	KDialogBase::slotApply();
}


void ConfigDialog::tmpPathBrowseClicked() {
	m_tmpPathLineEdit->setText(
			KFileDialog::getExistingDirectory(
		QString::null, this, "Select folder to store temporary files"));
}

void ConfigDialog::destPathBrowseClicked() {
	m_destPathLineEdit->setText(
			KFileDialog::getExistingDirectory(
		QString::null, this, "Select folder to store downloaded files"));
}

void ConfigDialog::addServerClicked() {
	ServerSettingsDialog s(this);
	connect(&s,SIGNAL(serverDetails(Server*)),this,SLOT(addServer(Server*)));
	s.exec();
}

void ConfigDialog::addServer(Server* newserver) {
	QValueListIterator<Server*> end = m_settings.serverList.end();
	for (QValueListIterator<Server*> it = m_settings.serverList.begin(); it!=end; it++) {
		if ((*it)->displayName()==newserver->displayName()) {
			QMessageBox::critical(this,"Knzb",i18n("A server with this name already exists\n"));
			return;
		}
	}
	m_settings.serverList.append(newserver);
	redrawServerList();
}

void ConfigDialog::redrawServerList() {
	for (QListViewItem *it = m_serverListView->firstChild(); it;) {
		QListViewItem *it2 = it;
		it = it->nextSibling();
		delete it2;
	}
	
	QValueListIterator<Server*> end = m_settings.serverList.end();
	for (QValueListIterator<Server*> it = m_settings.serverList.begin(); it!=end; it++) {
		new QListViewItem(m_serverListView,(*it)->displayName(), QString::number((*it)->priority()), 
				QString::number((*it)->maxThreads()));
	}
}

void ConfigDialog::modifyServerClicked() {
	//There's only one row selected, at most, due to selection mode
	QString serverName = m_serverListView->selectedItem()->text(0);
	QValueListIterator<Server*> end = m_settings.serverList.end();
	for (QValueListIterator<Server*> it = m_settings.serverList.begin(); it!=end; it++) {
		if ((*it)->displayName()==serverName) {
			ServerSettingsDialog s(this);
			connect(&s,SIGNAL(serverDetails(Server*)),this,SLOT(modifyServer(Server*)));
			s.modifyServerSettings((*it));
			s.exec();
			break;
		}
	}
}

void ConfigDialog::modifyServer(Server* changedServer) {
	//Find and delete the server, and replace it with ours.
	QValueListIterator<Server*> end = m_settings.serverList.end();
	for (QValueListIterator<Server*> it = m_settings.serverList.begin(); it!=end; it++) {
		if ((*it)->displayName()==changedServer->displayName()) {
			delete (*it);
			m_settings.serverList.remove(it);
			m_settings.serverList.append(changedServer);
			redrawServerList();
			return;
		}
	}
}

void ConfigDialog::deleteServerClicked() {
	//There's only one row selected, at most, due to selection mode
	QString serverName = m_serverListView->selectedItem()->text(0);
	delete m_serverListView->selectedItem();
	
	QValueListIterator<Server*> end = m_settings.serverList.end();
	for (QValueListIterator<Server*> it = m_settings.serverList.begin(); it!=end; it++) {
		if ((*it)->displayName()==serverName) {
			delete (*it);
			m_settings.serverList.remove(it);
			return;
		}
	}
}

void ConfigDialog::serverSelectionChanged() {
	m_modifyServerButton->setEnabled(m_serverListView->selectedItem()!=NULL);
	m_removeServerButton->setEnabled(m_serverListView->selectedItem()!=NULL);
}

