/***************************************************************************
                          downloader.h  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOWNLOADER_H
#define DOWNLOADER_H

class QSocket;
class QTimer;
class QDir;
class QDataStream;
#include <qobject.h>
#include <qfile.h>
class NntpPost;
class Server;

class Downloader : public QObject {
Q_OBJECT
public: 
	Downloader(Server*, QString, QString);
	~Downloader();
	NntpPost* post() const;
	Server* server() const;
	void downloadPost(NntpPost*);

	//Messages
  	enum {
		//Happy messages
		CONNECTED,
		AUTHED,
		CHANGED_GROUP,
		REQUESTING,
		DOWNLOADING,
		COMPLETED
	};

	enum {
		//Unhappy messages
		ERR_CONN,
    		ERR_DNS,
    		ERR_CONN_REFUSED,
		ERR_TIMEOUT,
		ERR_SERVER_UNHAPPY,
    		ERR_CONNLIMIT,
		ERR_AUTH,
		ERR_NO_GROUP,
		ERR_NO_POST,
		ERR_TRANSFER
	};

	enum {
		//Progress
		PERCENT,
		SPEED
	};
	
signals:

	void message(Downloader*, int);
	void error(Downloader*, int);
	void progress(Downloader*, int, int);

private:
	enum {
		INITIAL_CONNECT,
		CHANGING_GROUP,
		AUTH_USERNAME,
		AUTH_PASSWORD,
		READY,
		REQUESTING_POST,
		DOWNLOADING_POST
	} m_state;

	void changeGroup();
	void loginUsername();
	void loginPassword();
	void requestPost();

	bool grabPost();
	void disconnect();

	int writeBlock(QString);
	QString readResponse();
	

	QString m_postPrefix;
	//Status flag
	bool m_loggedIn; 

	QSocket *m_socket;
	NntpPost *m_post;
	Server *m_server;

	QTimer *m_timeoutTimer;
	QTimer *m_speedCalcTimer;

	QFile m_postFile;
	QDataStream m_postStream;

	int m_bytesDown, m_previousBytesDown;
	int m_previousAnnouncedPercentage;
	int m_speedRate, m_speedCount;

	int m_speeds[5];

	QString m_currentGroup;
	QString m_tempDir;

	bool m_inSockSignal;

	
	private slots:
	virtual void socketConnectedOk();
	virtual void socketError(int);
	virtual void dataReady();
	virtual void timeOut();
	virtual void speedCalc();
};
#endif
