/***************************************************************************
                          downloadmanager.h  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOWNLOADMANAGER_H
#define DOWNLOADMANAGER_H

#include <qobject.h>
#include <qptrlist.h>
#include <qvaluelist.h>
#include <qguardedptr.h>

class QString;
class QStringList;

class Downloader;
class ServerManager;
class Server;
class NntpPost;
class File;
class Decoder;
class Settings;
/**
  *@author David Pye
  */

class DownloadManager : public QObject  {
Q_OBJECT 
public:
	DownloadManager();
	~DownloadManager();

	//Queue control
	void startQueue();
	void stopQueue();
	
	bool downloadInProgress();
	
	void applySettings(Settings *);

	enum {
		SERVER_ERR_NONE_AVAILABLE,
		SERVER_ERR_DNS,
		SERVER_ERR_CONN_REFUSED,
		SERVER_ERR_TIMEOUT,
		SERVER_ERR_UNKNOWN,
		POST_ERR_UNOBTAINABLE,
		TEMP_DIR_NOT_WRITABLE,
		DEST_DIR_NOT_WRITABLE
	};

private:

	void startNewDownloader(Server*, NntpPost*);
	void killDownloader(Downloader*);
	Decoder *getDecoderForPost(QStringList parts);
	NntpPost *getNextPostInQueue();
	
	void downloaderCompleted(Downloader*);

	ServerManager *m_serverManager;
	QPtrList<Downloader> m_activeDownloaders;
	
	QString m_postPrefix;

	QGuardedPtr<File> m_currentFile;

	QValueList<File*> fileDecodeQueue;

	//Settings loaded from config
	QString m_tempDir, m_destDir;
	bool m_skipDownloadedFiles;
	bool m_downloadInProgress;


private slots:
	virtual void decodeFile();
	//These are used to receive events from the downloaders.
	virtual void downloaderMessage(Downloader*,int);
	virtual void downloaderProgress(Downloader*,int,int);
	virtual void downloaderError(Downloader*,int);
	
signals:
	void threadStarted(int, QString);
	void threadConnected(int);
	void threadChangedGroup(int);
	void threadDownloading(int,int);//percentage.
	void threadSpeed(int,int);//kb/sec
	void threadDownloadStarted(int, QString);//post name
	void threadDownloaded(int);
	void threadFinished(int);

	void fileStateChanged(File*);

	void error(int, QString);
	void complete();
	
	void needFile(File**);
};

#endif
