/***************************************************************************
                          file.cpp  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>
#include <qdir.h>

#include <kdebug.h>

#include "file.h"
#include "nntppost.h"

File::File() : QObject() {
	m_totalSegments=0;
	m_sizeKB=0;
	m_readyToDecode = false;
	m_existsOnDisk = false;
}

File::~File(){
	//The posts are automatically destroyed when we are deleted.
}

void File::deletePostFiles(QString tempDir, QString postPrefix) {
	//Delete any already downloaded post files that correspond with this file.
	QDir d(tempDir);
	QValueListIterator<NntpPost*> end = m_postList.end();
	for (QValueListIterator<NntpPost*> p = m_postList.begin(); p!=end; ++p) {
		d.remove(postPrefix+(*p)->title(),false);
	}
}

void File::setPoster(QString tmpposter) {
	m_poster = tmpposter;
}

void File::setSubject(QString tmpsubject) {
	m_subject = tmpsubject;
}

void File::setGroup(QString tmpgroup) {
	m_group = tmpgroup;                        
}
void File::setState(int s) {
	m_state = s;
}

int File::state() {
	return m_state;
}

void File::setfilename(QString filename) {
	m_filename = filename;
}

void File::setTotalSegments(int tmptotalSegments) {
	m_totalSegments = tmptotalSegments;
}

QString File::subject() const {
	return m_subject;
}

QString File::group() const {
	return m_group;
}

QString File::filename() {
	if (m_filename.isEmpty()) {
		//We don't have a filename set- try to guess one from subject.
		int firstQuote =  m_subject.find("\"");
		int secondQuote =  m_subject.find("\"",firstQuote+1);
		//No pair of quotes - can't guess a name
		if (firstQuote==-1 || secondQuote==-1) m_filename = m_subject;
		//There are quotes
		else m_filename = m_subject.mid(firstQuote+1,secondQuote-firstQuote-1);
	}
	return m_filename;
}

int File::totalSegments() const {
	return m_totalSegments;
}

unsigned long File::sizeKB() const {
	return m_sizeKB;
}

bool File::existsOnDisk(QString dir) {
	if (m_existsOnDisk) return true;
	
	QFile f(dir+'/'+filename());
	if (f.exists()) { 
		m_existsOnDisk=true;
		return true;
	}	
	else {
		return false;
	}	
}

bool File::readyToDecode() {
	//No posts are queued or downloading if this returns true.
	//Some posts may be permanently missing, however.
	if (m_readyToDecode) return true;
	QValueListIterator<NntpPost*> end = m_postList.end();
	for (QValueListIterator<NntpPost*> p = m_postList.begin(); p != end; p++) {
		if ((*p)->state()==NntpPost::DOWNLOADING || (*p)->state()==NntpPost::QUEUED ||
				(*p)->state()==NntpPost::WAITING) {
     			//kdDebug()<<"Segments "<<QString::number((*p)->sequenceNumber())<<" of file "<<
			//	m_filename<<" isnt complete - state is "<<(*p)->state()<<"\n";
			return false;
		}
	}
	//kdDebug()<<"File: This file is complete\n";

	return (m_readyToDecode=true);
}

void File::addPost(NntpPost *p) {
	connect(p, SIGNAL(destroyed()), this, SLOT(postDeleted()));
	m_postList.append(p);
	m_sizeKB+=p->bytes()/1024;
}

NntpPost* File::nextNeededPost() {
	if (readyToDecode()) {
		kdDebug()<<"File "<<m_filename<<" has no more needed posts\n";
		return NULL;
	}
	QValueListIterator<NntpPost*> end = m_postList.end();
	for (QValueListIterator<NntpPost*> p=m_postList.begin(); p!=end; p++) {
		if ((*p)->state()==NntpPost::WAITING) {
			return (*p);
		}
	}
	return NULL;
}

QValueList<NntpPost*> File::posts() const {
	return m_postList;
}

void File::postDeleted() {
        const NntpPost *p = dynamic_cast<const NntpPost*>(QObject::sender());
        if (p) m_postList.remove((NntpPost*)p);
}
