/***************************************************************************
                          knzb.cpp  -  description
                             -------------------
    begin                : Wed Jun 23 23:02:12 BST 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <kstatusbar.h>
#include <ksystemtray.h>
#include <kaction.h>
#include <kfiledialog.h>
#include <kstdaction.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <dcopclient.h>
#include <dcopobject.h>

#include <qmessagebox.h>
#include <qfileinfo.h>

#include "knzb.h"
#include "file.h"
#include "nzbparser.h"
#include "mainwidget.h"
#include "configdialog.h"
#include "settings.h"

Knzb::Knzb(QWidget *parent, const char *name) : DCOPObject("Knzb"),KMainWindow(parent, name) {

	setAutoSaveSettings("Main window",true);

	m_systemTray=NULL;
	m_quitApp=false;
	
	mainWidget = new MainWidget(this);
	mainWidget->setSettings(&m_settings);
	
	setCentralWidget(mainWidget);

	//File menu items
	KStdAction::open(this, SLOT(openFileDialog()), actionCollection());
  	KStdAction::quit(this, SLOT(quitApp()), actionCollection());

	//Queue control menu items
	(void)new KAction(i18n("St&art queue"), "1rightarrow.png", 0, this, SLOT(startQueue()),
		                        actionCollection(), "start_queue");
	(void)new KAction(i18n("&Pause queue"), "player_pause.png", 0, this, SLOT(pauseQueue()),
		                        actionCollection(), "pause_queue");
	(void)new KAction(i18n("&Empty queue"), "edittrash.png", 0, this, SLOT(emptyQueue()),
		                        actionCollection(), "empty_queue");

	(void)new KAction(i18n("Move selected &up"), "up.png", 0, mainWidget, SLOT(moveUp()),
		                        actionCollection(), "move_up");
	(void)new KAction(i18n("Move selected &down"), "down.png", 0, mainWidget, SLOT(moveDown()),
		                        actionCollection(), "move_down");
	(void)new KAction(i18n("Move selected to &top"), "top.png", 0, mainWidget, SLOT(moveTop()),
		                        actionCollection(), "move_top");
	(void)new KAction(i18n("Move selected to &bottom"), "bottom.png", 0, mainWidget, SLOT(moveBottom()),
		                        actionCollection(), "move_bottom");
	(void)new KAction(i18n("&Remove selected"), "remove.png", 0, mainWidget, SLOT(remove()),
		                        actionCollection(), "remove");

	//Configure menu
	KStdAction::preferences(this, SLOT(customize()), actionCollection());
	
	//createGUI("/tmp/knzb/knzb/knzbui.rc");
	createGUI();
	
	//Check specified temp and download directories exist
	connect(mainWidget, SIGNAL(statusBarMessage(QString)),this,SLOT(statusBarMessage(QString)));
	connect(mainWidget, SIGNAL(queueSize(double)),this,SLOT(queueSize(double)));
		
	checkDirsExist();
	
	applySettings();

	m_queueSizeLabel = new QLabel(this);
	m_queueSizeLabel->setText(i18n("Queue size:")+"0MB");
	statusBar()->addWidget(m_queueSizeLabel, 0, true);
	statusBar()->message(i18n("Welcome to knzb!"),5000);
	
	//Register with DCOP
	if (!kapp->dcopClient()->isRegistered()) {
		kapp->dcopClient()->registerAs("Knzb");
		kapp->dcopClient()->setDefaultObject(objId());
	}
}

Knzb::~Knzb() {
	delete mainWidget;
	m_settings.saveSettings();
}

void Knzb::statusBarMessage(QString msg) {
	statusBar()->message(msg);
}

void Knzb::queueSize(double newSizeMB) {
	m_queueSizeLabel->setText(i18n("Queue size: ") + QString::number(newSizeMB,'f',2) + "MB");
}

void Knzb::applySettings() {
	//Reload settings.
	m_settings.loadSettings();
	if (m_settings.dockToSystray && m_systemTray==NULL) {
		KIconLoader l("knzb");
		m_systemTray = new KSystemTray(this, "Knzb");
		m_systemTray->setPixmap(l.loadIcon("connect_creating.png",KIcon::Toolbar));
		connect(m_systemTray, SIGNAL(quitSelected()), this, SLOT(quitApp()));
		m_systemTray->show();
	}
	else if (!m_settings.dockToSystray && m_systemTray!=NULL) {
		delete m_systemTray;
		m_systemTray=NULL;
	}
	mainWidget->applySettings();
}

void Knzb::openFileDialog() {
	QStringList nzbFilenames =  KFileDialog::getOpenFileNames("","*.nzb|NZB files (*.nzb)",this, i18n("Load NZB file"));

	QStringList::iterator end = nzbFilenames.end();
	for (QStringList::iterator it=nzbFilenames.begin(); it!=end; ++it) {
		if (!(*it).isEmpty()) {
			openFile(*it);
		}
	}
}

void Knzb::openFile(const QString &filename) {
	NzbParser nzbParser;
	QValueList<File*> nzb = nzbParser.parseFile((filename));

	
	if (!nzb.isEmpty()) {
		QFileInfo qf(filename);
		mainWidget->addDownload(qf.baseName(true), nzb);
	}
	else QMessageBox::critical(this,"Knzb",i18n("Unable to load/parse NZB file ")+(filename)+i18n(". Perhaps it is corrupt?"));
}

void Knzb::customize() {
	ConfigDialog dl(this);
	connect(&dl, SIGNAL(applyClicked()), this, SLOT(applySettings()));
	connect(&dl, SIGNAL(okClicked()), this, SLOT(applySettings()));
	dl.exec();
}

void Knzb::startQueue() {
	mainWidget->startQueue();	
}

void Knzb::pauseQueue() {
	mainWidget->pauseQueue();	
}

void Knzb::emptyQueue() {
	mainWidget->emptyQueue();	
}

void Knzb::checkDirsExist() {
	Settings s;
	s.loadSettings();
	QDir t(s.tempDir);
	QDir d(s.destDir);
	
	if (s.tempDir=="NONE" || s.destDir=="NONE" || !d.exists() || !t.exists()) {
		kdDebug()<<"Knzb: Temp/Dest directories undefined\n";
		QMessageBox::information(this,"Knzb","Before you can use knzb, you must configure at least one server and the directories to use to store temporary and completed files");
		customize();	
	}
}

bool Knzb::queryClose() {
	if (m_settings.dockToSystray && !m_quitApp) {
		//Dock into the systemtray if we were X'd from the window manager. 
		//You must close the app via File->Close to succeed.
		if (KMessageBox::questionYesNo (NULL, i18n("Please note: Closing the application window will make\
						the application dock to the system tray. To quit Knzb, please select Exit\
						from the File menu.   To alter this behaviour, you may enable or disable\
						system tray docking from the Configure dialog."), i18n("Systray docking"), KStdGuiItem::ok(), 
				KStdGuiItem::cancel(), "Don't warn me about docking in future")==KMessageBox::Yes) {
			hide();
		}
		return false;
	}
	else return true;
}

void Knzb::quitApp() {
	m_quitApp = true;
	close();
}
