/***************************************************************************
                          mainwidget.cpp  -  description
                             -------------------
    begin                : Wed Jun 23 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 * Something to watch for - several things in this class depend on the iterator
 * on a QListView traversing top to bottom - Nowhere does this behaviour seem
 * to be guaranteed...
 */

#include <qobject.h>
#include <qstatusbar.h>
#include <qpopupmenu.h>
#include <qtooltip.h>
#include <qprogressbar.h>
#include <qtable.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qlayout.h>

#include <klistview.h>
#include <kapplication.h>
#include <klocale.h>
#include <kdebug.h>

#include "mainwidget.h"
#include "downloadmanager.h"
#include "file.h"
#include "settings.h"
#include "fileviewitem.h"

MainWidget::MainWidget(QWidget* parent):QWidget(parent){
	m_settings=NULL;	
	m_queueSizeMB=0;

	QSplitter *splitter = new QSplitter(this);
	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->add(splitter);
	splitter->setOrientation(QSplitter::Vertical);

	fileView = new KListView(splitter, "File View");
	fileView->addColumn(i18n("Item"));
	fileView->addColumn(i18n("Status"));
	fileView->addColumn(i18n("Size (MB)"));
	fileView->addColumn(i18n("Segments"));
	fileView->setColumnWidthMode(0,QListView::Manual);
	fileView->setColumnWidthMode(1,QListView::Manual);
	fileView->setColumnWidthMode(2,QListView::Manual);
	fileView->setColumnWidthMode(3,QListView::Manual);
	
	fileView->setRootIsDecorated(true);
        fileView->setSorting(-1);
	fileView->setSelectionMode(QListView::Extended);
	fileView->restoreLayout(kapp->config(),"Main widget geometry");
	
	threadTable = new QTable(0,4,splitter);
	threadTable->setReadOnly(true);
		
	threadTable->horizontalHeader()->setLabel( 0, i18n("Server"));
	threadTable->horizontalHeader()->setLabel( 1, i18n("Post"));
	threadTable->horizontalHeader()->setLabel( 2, i18n("% Complete"));
	threadTable->horizontalHeader()->setLabel( 3, i18n("Status"));

	threadTable->setColumnStretchable(0,true);
	threadTable->setColumnStretchable(1,true);
	
	threadTable->setMaximumHeight((threadTable->numRows()+2) * threadTable->horizontalHeader()->height());

	m_downloadManager = new DownloadManager();


	connect(fileView,SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)),this,SLOT(popupMenuRequested(QListViewItem *, const QPoint &, int)));
	
	connect(m_downloadManager,SIGNAL(threadStarted(int,QString)),this,SLOT(threadStarted(int,QString)));
	connect(m_downloadManager,SIGNAL(threadConnected(int)),this,SLOT(threadConnected(int)));
	connect(m_downloadManager,SIGNAL(threadChangedGroup(int)),this,SLOT(threadChangedGroup(int)));
	connect(m_downloadManager,SIGNAL(threadDownloading(int,int)),this,SLOT(threadDownloading(int,int)));
	connect(m_downloadManager,SIGNAL(threadSpeed(int,int)),this,SLOT(threadSpeed(int,int)));
	connect(m_downloadManager,SIGNAL(threadFinished(int)),this,SLOT(threadFinished(int)));
	connect(m_downloadManager,SIGNAL(threadDownloadStarted(int, QString)),this,SLOT(threadDownloadStarted(int, QString)));

	connect(m_downloadManager,SIGNAL(fileStateChanged(File*)), this, SLOT(fileStateChanged(File*)));
	connect(m_downloadManager,SIGNAL(error(int, QString)), this,SLOT(downloadError(int, QString)));
	connect(m_downloadManager,SIGNAL(complete()), this,SLOT(complete()));
	connect(m_downloadManager,SIGNAL(needFile(File**)),this,SLOT(nextFile(File**)));
}


MainWidget::~MainWidget() {
	delete m_downloadManager;
	//Save the layout of the filetable.
	fileView->saveLayout(kapp->config(),"Main widget geometry");
}

void MainWidget::setSettings(Settings *s) {
	m_settings = s;
}

void MainWidget::applySettings() {
	//Load the setting which defines whether we delete completed items or not
	m_removeCompletedItemsFromQueue = m_settings->removeCompletedItemsFromQueue;
	m_treePerNzb = m_settings->treePerNzb;
	
	m_downloadManager->applySettings(m_settings);
}

void MainWidget::addDownload(QString name, QValueList<File*> files) {
	//Create a new parent node, named after the nzb download name, and add all the files
	//as child nodes of it.

	double newDownloadSizeMB=0;

	KListViewItem *parentNode=NULL;
	if (m_treePerNzb) {
		parentNode = new KListViewItem(fileView);
		parentNode->setText(0, name);
		
		//Move the parent download to the bottom of the list.
		QListViewItem *previousDownload;
		for (previousDownload = fileView->firstChild(); previousDownload->nextSibling(); previousDownload=previousDownload->nextSibling()) 
			;
		fileView->moveItem(parentNode, NULL, previousDownload);
	}
		
	//Add the child items in the right order - need to find a way to use sort really.
	QListViewItem *previousItem = NULL;

	for (QValueListIterator<File*> it = files.begin(); it!=files.end(); it++) {
		FileViewItem *item;
		if (m_treePerNzb) {
			//Attach this node to a subtree
			item = new FileViewItem(parentNode);
			fileView->moveItem(item,parentNode,previousItem);
		}
		else {
			//Toplevel
			item = new FileViewItem(fileView);	
			fileView->moveItem(item,NULL,previousItem);
		} 
		item->setText(0,(*it)->filename());
		item->file = *it;
		file2itemMap[*it] = item;
		(*it)->setState(File::QUEUED);
		item->setText(1,i18n("Queued"));
		QString size = QString::number((*it)->sizeKB()/1024.0,'f',2);
		item->setText(2,size);
		item->setText(3,QString::number((*it)->totalSegments()));
		
		newDownloadSizeMB+=(*it)->sizeKB()/1024.0;
		
		//Put the items in the right order. The default behaviour is newest at top,
		//which is not what we want.
		previousItem = item;
	}
	
	if (parentNode) parentNode->setText(2,QString::number(newDownloadSizeMB,'f',2));
	
	m_queueSizeMB+=newDownloadSizeMB;
	emit queueSize(m_queueSizeMB);
}

QString MainWidget::fileStateName(int state) {
	switch (state) {
		case File::QUEUED:
			return i18n("Queued");
		case File::DOWNLOADING:
			return i18n("Downloading");
		case File::DECODING:
			return i18n("Decoding");
		case File::COMPLETE:
			return i18n("Complete");
		case File::EXISTS:
			return i18n("Exists");
		case File::DECODE_ERROR:
			return i18n("Decoding error");
		case File::INCOMPLETE:
			return i18n("Incomplete");
		default:
			return i18n("Unknown-bug?");
	}
}

void MainWidget::startQueue() {
	if (!m_downloadManager->downloadInProgress() && m_queueSizeMB>0) {
		//Need to prod the download manager.
		m_downloadManager->startQueue();
	
		emit statusBarMessage(i18n("Queue started"));
	}
}

void MainWidget::downloadError(int err, QString data) {
	QString str;
	switch (err) {
		case DownloadManager::SERVER_ERR_NONE_AVAILABLE:
			str+=i18n("No priority 1 servers available");
			break;
		case DownloadManager::SERVER_ERR_DNS:
			str+=i18n("Unable to resolve server: ");
			str+=data;
			break;
		case DownloadManager::SERVER_ERR_CONN_REFUSED:
			str+=i18n("Connection refused: ");
			str+=data;
			break;
		case DownloadManager::SERVER_ERR_TIMEOUT:
			str+=i18n("Timeout connecting to server: ");
			str+=data;
			break;
		case DownloadManager::SERVER_ERR_UNKNOWN:
			str+=i18n("Unknown server error: ");
			str+=data;
			break;
		case DownloadManager::POST_ERR_UNOBTAINABLE:
			str+=i18n("Unable to find post ");
			str+=data;
			break;
		case DownloadManager::TEMP_DIR_NOT_WRITABLE:
			str+=i18n("Unable to write to temporary directory");
			break;
		case DownloadManager::DEST_DIR_NOT_WRITABLE:
			str+=i18n("Unable to write to destination directory");
			break;
		default:
			str+=i18n("Unknown error - bug?");
			break;
	}
	emit statusBarMessage(str);
}


void MainWidget::pauseQueue() {
	if (m_downloadManager->downloadInProgress()) {
		m_downloadManager->stopQueue();

		QListViewItemIterator it( fileView );
		while ( it.current() ) {
			FileViewItem *flItem = dynamic_cast<FileViewItem*>(it.current());
			if (flItem) {
				File *f = flItem->file;
				if (f && f->state()==File::DOWNLOADING) {
					flItem->setText(1,i18n("Queued"));
					f->setState(File::QUEUED);
				}
			}
			++it;
		}
		emit statusBarMessage(i18n("Queue paused"));
	}
}

void MainWidget::emptyQueue() {
	m_downloadManager->stopQueue();
	
	QListViewItemIterator it( fileView );
	while ( it.current() ) {
		QListViewItem *item  = it.current();
		FileViewItem *flItem = dynamic_cast<FileViewItem*>(item);
		if (flItem) {
			delete flItem->file;
			flItem->file=NULL;
		}
		++it;
	}
	fileView->clear();
	
	m_queueSizeMB=0;
	emit queueSize(0);
}

void MainWidget::deleteFileItem(FileViewItem *flItem) {
	file2itemMap.remove(flItem->file);
	
	QListViewItem *parent = flItem->parent();

	if (parent) parent->setText(2,
		QString::number(parent->text(2).toDouble() - flItem->file->sizeKB()/1024.0,'f',2));

	m_queueSizeMB -= flItem->file->sizeKB()/1024.0;
	
	delete flItem;

	if (parent && parent->firstChild()==NULL) delete parent;
}

void MainWidget::nextFile(File** file) {
	//A bit clunky - this depends on the iterator traversing the list top to bottom
	//The Qt api does not appear to guarantee this.
	
	QListViewItemIterator it( fileView );
	while ( it.current() ) {
		QListViewItem *item  = it.current();
		FileViewItem *flItem = dynamic_cast<FileViewItem*>(item);
		if (flItem) {
			File *f = flItem->file;
			if (f && f->state() == File::QUEUED) {
				*file = f;
				return;
			}
		}
		++it;
	}
	//No more files are queued.
	*file=NULL;
}

void MainWidget::moveTop() {
	QPtrList<QListViewItem> selectedItems = fileView->selectedItems();
	for (QListViewItem *itemIt = selectedItems.last(); itemIt; itemIt=selectedItems.prev()) {
		if (itemIt->parent()) fileView->moveItem(itemIt, itemIt->parent(), NULL);	
		else fileView->moveItem(itemIt, NULL, NULL);
	}
	
	if (m_downloadManager->downloadInProgress()) {
		pauseQueue();
		startQueue();
	}
}

void MainWidget::moveBottom() {
	QPtrList<QListViewItem> selectedItems = fileView->selectedItems();
	for (QListViewItem *itemIt = selectedItems.first(); itemIt; itemIt=selectedItems.next()) {
		if (itemIt->parent()) {
			QListViewItem *lastChild;
			for (lastChild = itemIt->parent()->firstChild(); lastChild && lastChild->nextSibling(); 
					lastChild = lastChild->nextSibling())
				;
			fileView->moveItem(itemIt, itemIt->parent(), lastChild);	
		}
		else fileView->moveItem(itemIt, NULL, fileView->lastChild());
	}
	
	if (m_downloadManager->downloadInProgress()) {
		pauseQueue();
		startQueue();
	}
}

void MainWidget::moveUp() {
	bool dlWasInProgress=m_downloadManager->downloadInProgress();
	if (dlWasInProgress) {
		pauseQueue();
	}

	QPtrList<QListViewItem> selectedItems = fileView->selectedItems();
	for (QListViewItem *item = selectedItems.first(); item; item = selectedItems.next()) {
		if (item->isSelected() && item->itemAbove()) {
			//It's selected and there is an item above it.
			QListViewItem *currentItemAbove = item->itemAbove();
			for (QListViewItem *newAbove = item->itemAbove()->itemAbove(); ; newAbove = newAbove->itemAbove()) {
				//If the item above shares the same parent as us, or is null,
				//move us.
				if (!newAbove || newAbove->parent() == item->parent()) {
					fileView->moveItem(item, item->parent(), newAbove);
				}
				
				//If we actually ended up being moved, or there are no more items
				//above, break. (Complicated to explain!). Stupid widget.
				if (!newAbove || (item->itemAbove() != currentItemAbove)) break;
			}
		}
	}

	//If we were downloading, restart the queue.
	if (dlWasInProgress) {
		startQueue();
	}
}


void MainWidget::moveDown() {
	bool dlWasInProgress=m_downloadManager->downloadInProgress();
	if (dlWasInProgress) {
		pauseQueue();
	}

	QPtrList<QListViewItem> selectedItems = fileView->selectedItems();
	for (QListViewItem *item = selectedItems.last(); item; item = selectedItems.prev()) {
		if (item->isSelected()) {
			//It's selected and there is an item above it.
			QListViewItem *currentItemAbove = item->itemAbove();
			for (QListViewItem *newBelow = item->itemBelow() ; newBelow ; newBelow = newBelow->itemBelow()) {
				//If the item above shares the same parent as us, or is null,
				//move us.
				if (newBelow->parent() == item->parent()) {
					fileView->moveItem(item, item->parent(), newBelow);
					//If we actually moved, break.
					if (item->itemAbove() != currentItemAbove) break;
				}
			}
		}
	}

	//If we were downloading, restart the queue.
	if (dlWasInProgress) {
		startQueue();
	}
}


void MainWidget::remove() {
	QListViewItemIterator it( fileView );
	bool restartQueue=false;
	
	while ( it.current() ) {
		if (it.current()->isSelected()) {
			//This is a file item	
			FileViewItem *flItem = dynamic_cast<FileViewItem*>(it.current());
			if (flItem) {
				if (flItem->file->state()==File::DOWNLOADING) {
					restartQueue=true;
					m_downloadManager->stopQueue();
				}
				deleteFileItem(flItem);
			}
			else {
				//This is a parent item. Delete all its children.
				flItem = dynamic_cast<FileViewItem*>(it.current()->firstChild());
				while (flItem) {
					if (flItem->file->state()==File::DOWNLOADING) {
						restartQueue=true;
						m_downloadManager->stopQueue();
					}
					FileViewItem *nextItem = dynamic_cast<FileViewItem*>(flItem->nextSibling());
					deleteFileItem(flItem);
					flItem = nextItem;
				}
			}
		}
		//If this item was selected, don't move the iterator, as it's now been deleted.
		//Otherwise, shift to the next one.
		else ++it;

	}

	if (restartQueue && fileView->firstChild()!=NULL ) m_downloadManager->startQueue();

	//Update the queue size.
	emit queueSize(m_queueSizeMB);
}

void MainWidget::fileStateChanged(File *f) {
	QMap<File*, FileViewItem*>::Iterator it = file2itemMap.find(f);
	if (it == file2itemMap.end()) {
		kdDebug()<<"MainWidget: Item for file "<<f->filename()<<" not found in file2itemMap - probable bug!\n";
		return;
	}
	
	FileViewItem *item = (*it);

	QListViewItem *parentNode = item->parent();
	
	switch (f->state()) {
		case File::COMPLETE:
		case File::EXISTS:
			
			if (m_removeCompletedItemsFromQueue) {
				//Delete this item and its parent, if we have one, and
				//if we are the only item it contains
				deleteFileItem(item);
			}
			else {
				file2itemMap.remove(f);
				QListViewItem *parent = item->parent();

				if (parent) parent->setText(2,
					QString::number(parent->text(2).toDouble() - f->sizeKB()/1024.0,'f',2));

				m_queueSizeMB -= f->sizeKB()/1024.0;
				
				item->setText(1,fileStateName(f->state()));
			}

			emit queueSize(m_queueSizeMB);
			break;
		default:
			//Reset filename, in case it was changed.
			item->setText(0, f->filename());
			item->setText(1,fileStateName(f->state()));
	}
}

void MainWidget::complete() {
	emit statusBarMessage(i18n("Queue complete"));
}

void MainWidget::popupMenuRequested(QListViewItem *item, const QPoint & point, int) {
	QPopupMenu *popupMenu = new QPopupMenu();
	int topId = popupMenu->insertItem(i18n("Move selected to top"),this,SLOT(moveTop()));
	int bottomId = popupMenu->insertItem(i18n("Move selected to bottom"),this,SLOT(moveBottom()));
	int upId = popupMenu->insertItem(i18n("Move selected up"),this,SLOT(moveUp()));
	int downId = popupMenu->insertItem(i18n("Move selected down"),this,SLOT(moveDown()));
	popupMenu->insertSeparator();
	int removeId = popupMenu->insertItem(i18n("Remove selected"),this,SLOT(remove()));
	
	if (fileView->selectedItems().isEmpty()) {
		popupMenu->setItemEnabled(topId, false);
		popupMenu->setItemEnabled(bottomId, false);
		popupMenu->setItemEnabled(upId, false);
		popupMenu->setItemEnabled(downId, false);
		popupMenu->setItemEnabled(removeId, false);
	}
		
	popupMenu->exec(point);
	delete popupMenu;
}


//Thread updates

void MainWidget::threadStarted(int id, QString serverName) {
	if (threadTable->numRows()-1 < id) threadTable->setNumRows(id+1);
	//Increment the table size
	threadTable->setMaximumHeight((threadTable->numRows()+2) * threadTable->horizontalHeader()->height());
	
	threadTable->setCellWidget(id,2,new QProgressBar(threadTable));
	threadTable->setText(id,0,serverName);
	threadTable->setText(id,3,i18n("Connecting"));
	
	QProgressBar *progBar = dynamic_cast<QProgressBar*>(((QProgressBar*)threadTable->cellWidget(id,2)));
	if (progBar) {
		progBar->setProgress(0);
	}
}

void MainWidget::threadConnected(int id) {
	threadTable->setText(id,3,i18n("Connected"));
}

void MainWidget::threadChangedGroup(int) {
	threadTable->setText(0,3,i18n("Changed group"));
}

void MainWidget::threadDownloading(int id, int percent) {
	QProgressBar *progBar = dynamic_cast<QProgressBar*>(((QProgressBar*)threadTable->cellWidget(id,2)));
	if (progBar) {
		progBar->setProgress(percent);
	}
}

void MainWidget::threadDownloadStarted(int id, QString postName) {
	threadTable->setText(id,3,i18n("Waiting"));
	threadTable->setText(id,1,postName);
	
	QProgressBar *progBar = dynamic_cast<QProgressBar*>(((QProgressBar*)threadTable->cellWidget(id,2)));
	if (progBar) {
		progBar->setProgress(0);
	}

}

void MainWidget::threadSpeed(int id, int speed) {
	threadTable->setText(id,3,i18n("Downloading")+' '+QString::number(speed)+"kB/s");
}

void MainWidget::threadFinished(int id) {
	threadTable->removeRow(id);
	//Decrement table maxsize.
	threadTable->setMaximumHeight((threadTable->numRows()+2) * threadTable->horizontalHeader()->height());
}

