/***************************************************************************
                          mainwidget.h  -  description
                             -------------------
    begin                : Wed Jun 23 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MAINWIDGET_H
#define MAINWIDGET_H

#include <qwidget.h>
#include <qmap.h>
class QTable;
class QProgressBar;
class QSplitter;
class QString;
class QListViewItem;

class KListView;

class DownloadManager;
class File;
class FileViewItem;
class Settings;
/**
  *@author David Pye
  */

class MainWidget : public QWidget  {
	Q_OBJECT
public: 
	MainWidget(QWidget *parent);
	~MainWidget();
	void addDownload(QString, QValueList<File*>);
	void setSettings(Settings*);
	void applySettings();
	
	//Returns a pointer to the first non-downloading file.
	File *getNextFile();
	
private:
	QString fileStateName(int);
	
	//GUI components
	QTable *threadTable;
	KListView *fileView;

	DownloadManager *m_downloadManager;
	//Settings,etc
	Settings *m_settings;

	bool m_removeCompletedItemsFromQueue, m_treePerNzb;
	
	QMap<File*, FileViewItem*> file2itemMap;

	double m_queueSizeMB;

	void deleteFileItem(FileViewItem*);

public slots:
	//Slots to respond to UI events.
	virtual void startQueue();
	virtual void pauseQueue();
	virtual void emptyQueue();
	
	virtual void moveTop();
	virtual void moveBottom();
	virtual void moveUp();
	virtual void moveDown();
	virtual void remove();

	virtual void popupMenuRequested(QListViewItem *, const QPoint &, int);
	
private slots:
	virtual void downloadError(int, QString);
	virtual void fileStateChanged(File*);
	virtual void nextFile(File **);
	
	virtual void threadStarted(int, QString);
	virtual void threadConnected(int);
	virtual void threadChangedGroup(int);
	virtual void threadDownloadStarted(int, QString);
	virtual void threadDownloading(int,int);//percentage.
	virtual void threadFinished(int);
	virtual void threadSpeed(int,int);
	virtual void complete();
	
	//Slots to respond to UI events.
signals:

	virtual void statusBarMessage(QString);
	virtual void queueSize(double);
};

#endif
