/***************************************************************************
                          nzbdata.cpp  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "nzbparser.h"

#include <qfileinfo.h>
#include <kapplication.h>
#include <file.h>
#include <qdom.h>
#include "file.h"
#include "nntppost.h"

NzbParser::NzbParser(){
}

NzbParser::~NzbParser(){
}

QValueList<File*> NzbParser::parseFile(QString fileName){
	QFile nzbfile(fileName);

	QValueList<File*> list;

	if (!nzbfile.open(IO_ReadOnly)) return list;
	
	QDomDocument doc( "nzbfile" );
	doc.setContent(&nzbfile);
	nzbfile.close();
	QFileInfo qf(fileName);
	
	QDomElement docElem = doc.documentElement();
	QDomNode n = docElem.firstChild();
	while( !n.isNull() ) {
		QDomElement e = n.toElement(); // try to convert the node to an element.
		kapp->processEvents();
		if( !e.isNull() ) {
			if (e.tagName()=="file") {
				//This is a file tag
				int numSegments=0;
				File *file = new File();
				file->setSubject(e.attribute("subject"));
				file->setPoster(e.attribute("poster"));
				//We ignore the date attribute - we don't need it.
				QDomNode fileSubNode = e.firstChild();
				while (!fileSubNode.isNull()) {
					kapp->processEvents();
					QDomElement fileElement = fileSubNode.toElement();
					if (!fileElement.isNull()) {
						if (fileElement.tagName()=="groups") {
							//Found a groups tag
							QDomNode groupsSubNode = fileSubNode.firstChild();
							while (!groupsSubNode.isNull()) {
								kapp->processEvents();
								QDomElement groupsElement = groupsSubNode.toElement();
								if (!groupsElement.isNull()) {
									if (groupsElement.tagName()=="group") {
										if (file->group().isEmpty()) file->setGroup(groupsElement.text());
									}
								}
								groupsSubNode = groupsSubNode.nextSibling();
							}
						}
						else if (fileElement.tagName()=="segments") {
							QDomNode segmentsSubNode = fileSubNode.firstChild();
							while (!segmentsSubNode.isNull()) {
								kapp->processEvents();
								//Process some events.
								QDomElement segmentsElement = segmentsSubNode.toElement();
								if (!segmentsElement.isNull()) {
									if (segmentsElement.tagName()=="segment") {
										NntpPost *post = new NntpPost(file);
										post->setTitle(segmentsElement.text());
										post->setBytes(segmentsElement.attribute("bytes").toInt());
										post->setSequenceNumber(segmentsElement.attribute("number").toInt());
										if (segmentsElement.attribute("number").toInt()>numSegments)
											numSegments = segmentsElement.attribute("number").toInt();
										post->setGroup(file->group());
										file->addPost(post);
									}
								}
								segmentsSubNode = segmentsSubNode.nextSibling();
							}
						}
					}
					fileSubNode = fileSubNode.nextSibling();
				}
				//kdDebug()<<"Adding file "+file->filename()+"\n";
				file->setTotalSegments(numSegments);
				list.append(file);
			}
		}
		n = n.nextSibling();
	}
	return list;
}    
