/***************************************************************************
                          servermanager.cpp  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qobject.h>
#include <qstringlist.h>

#include <kdebug.h>

#include "nntppost.h"
#include "servermanager.h"
#include "settings.h"
#include "server.h"


ServerManager::ServerManager(){

}

ServerManager::~ServerManager(){

}

void ServerManager::applySettings(Settings *s) {
	//This needs to be rethought in order to avoid massive problems with the 
	//QMap fro tried servers.  Maybe a list of server names rather than
	//pointers to servers would be less fragile.
	m_serverList = (s->serverList);	
}

Server* ServerManager::getNextServer(Server* server) {
	if (server==NULL && !m_serverList.isEmpty()) return *(m_serverList.begin());
	else {
		QValueListIterator<Server*> end = m_serverList.end();
		QValueListIterator<Server*> it = m_serverList.find(server);
		if (it!=end) {
			it++;
			for (; it!=end; ++it) {
				if ((*it)->enabled()) return (*it);
			}
		}
		//We did not find one.
		return NULL;
	}
}

Server* ServerManager::getNextServerForPost(NntpPost* p, Server *failedServer) {
	//We keep a record of the servers tried for this post in the QMap.
	//We connect to this posts's deleted signal, so we can remove our record
	//once the post is gone.
	
	connect(p, SIGNAL(destroyed()), this, SLOT(postDeleted()));
	
	//Add failedServer to the list of servers that have been tried for this post
	//and return the next one that isnt in the list
	
	Server *serverToReturn = NULL;

	QValueList<Server*> listOfTriedServers;
	
	//Look up the tried servers for this post, if present.
	QMap<const NntpPost*, QValueList<Server*> >::Iterator mapIt = m_serversTriedMap.find(p);
	if (mapIt != m_serversTriedMap.end()) {
		//Add the servers in the existing record to the new one.
		listOfTriedServers+=(*mapIt);
	}
	if (failedServer!=NULL) listOfTriedServers.append(failedServer);
	
	//If there are some tried servers for this post, insert the map entry.
	if (! listOfTriedServers.isEmpty()) m_serversTriedMap[p] = listOfTriedServers;

	QValueListIterator<Server*> end = m_serverList.end();
	for (QValueListIterator<Server*> it = m_serverList.begin(); it != end; ++it) {
		//If this server hasn't been tried, it's the one.
		if (listOfTriedServers.contains((*it))==0) {
			serverToReturn = (*it);		
			break;
		}	
	}
	return serverToReturn;
}


void ServerManager::postDeleted() {
	const NntpPost *p = dynamic_cast<const NntpPost*>(QObject::sender());
	if (p) m_serversTriedMap.remove(p);
}
