/***************************************************************************
                          serversettingsdialog.cpp  -  description
                             -------------------
    begin                : Wed Jul 7 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtooltip.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qframe.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qgroupbox.h>

#include <klocale.h>

#include "serversettingsdialog.h"
#include "server.h"

ServerSettingsDialog::ServerSettingsDialog(QWidget *parent, const char* name ): KDialogBase (Plain,
	i18n("News server configuration"), Ok|Cancel, Ok,parent,name,true) {

	QFrame *frame = plainPage();
	QVBoxLayout *layout = new QVBoxLayout( frame, 0, spacingHint() );

	QGroupBox *serverGb = new QGroupBox(2, Vertical, i18n("General"),frame,NULL);
	layout->addWidget(serverGb);
	QWidget *generalWidget = new QWidget(serverGb);
	QGridLayout *generalLayout = new QGridLayout(generalWidget,7,2);
	generalLayout->addWidget(new QLabel(i18n("Display name:"),generalWidget),0,0);
	m_displayName = new QLineEdit(generalWidget);
	QToolTip::add(m_displayName,i18n("The name used to identify this server"));
	generalLayout->addWidget(m_displayName,0,1);

	generalLayout->addWidget(new QLabel(i18n("Host name:"),generalWidget),1,0);
	m_hostname = new QLineEdit(generalWidget);
	QToolTip::add(m_hostname,i18n("The DNS hostname, or IP address of the server"));
	generalLayout->addWidget(m_hostname,1,1);
	
	generalLayout->addWidget(new QLabel(i18n("Priority:"),generalWidget),2,0);
	m_priority = new QSpinBox(1,10,1,generalWidget);
	QToolTip::add(m_priority,i18n("The priority ranking of the server - lowest rated servers are used in preference"));
	generalLayout->addWidget(m_priority,2,1);           
	
	generalLayout->addWidget(new QLabel(i18n("TCP/IP Port:"),generalWidget),3,0);
	m_portNo = new QLineEdit(generalWidget);
	m_portNo->setText("119");
	QToolTip::add(m_portNo,i18n("The TCP/IP port to connect to - 119 is the default for NNTP"));
	generalLayout->addWidget(m_portNo,3,1);  

  	generalLayout->addWidget(new QLabel(i18n("Timeout(seconds):"),generalWidget),4,0);
	m_timeout = new QSpinBox(1,120,5,generalWidget);
	m_timeout->setValue(30);
	QToolTip::add(m_timeout,i18n("The timeout in seconds for this server. If exceeded, the thread will reconnect"));
	generalLayout->addWidget(m_timeout,4,1);   

	generalLayout->addWidget(new QLabel(i18n("Maximum connections:"),generalWidget),5,0);
	m_maxConnections = new QSpinBox(1,50,1,generalWidget);
	QToolTip::add(m_maxConnections,i18n("The maximum number of simultaneous connections this server allows"));
	generalLayout->addWidget(m_maxConnections,5,1);           
	
	m_enabled = new QCheckBox(i18n("Enabled"),generalWidget);
	m_enabled->setChecked(true);
	QToolTip::add(m_enabled,i18n("Whether the server is available for use"));
	generalLayout->addWidget(m_enabled,6,0);
		
	QGroupBox *authGb = new QGroupBox(2,Vertical,i18n("Authentication"),frame,NULL);
	layout->addWidget(authGb);                        	
	m_authRequired  = new QCheckBox(i18n("Authentication required"),authGb);
	QToolTip::add(m_authRequired,i18n("Whether the server requires a username and password"));
	
	QWidget *authWidget = new QWidget(authGb);
	QGridLayout *authLayout = new QGridLayout(authWidget,2,2);
	authLayout->addWidget(new QLabel(i18n("Username:"),authWidget),0,0);
	m_username = new QLineEdit(authWidget);
	QToolTip::add(m_username,i18n("The username to use when connecting to this server"));
	authLayout->addWidget(m_username,0,1);
	
	authLayout->addWidget(new QLabel(i18n("Password:"),authWidget),1,0);
	m_password = new QLineEdit(authWidget);
	m_password->setEchoMode(QLineEdit::Password);
	QToolTip::add(m_password,i18n("The password to use when connecting to this server"));
	authLayout->addWidget(m_password,1,1);

	m_username->setEnabled(false);
	m_password->setEnabled(false);
	connect(m_authRequired,SIGNAL(toggled(bool)),this,SLOT(authRequiredToggled(bool)));
}

ServerSettingsDialog::~ServerSettingsDialog(){
}

void ServerSettingsDialog::slotOk() {
	if (checkValid()) {
		Server *server = new Server();
		server->setDisplayName(m_displayName->text());
		server->setHostName(m_hostname->text());
		server->setPriority(m_priority->text().toInt());
		server->setMaxThreads(m_maxConnections->text().toInt());
		server->setTimeout(m_timeout->value());
		server->setAuthRequired(m_authRequired->isChecked());
		server->setEnabled(m_enabled->isChecked());
		
		server->setUsername(m_username->text());
		server->setPassword(m_password->text());
		server->setPort(m_portNo->text().toInt());
		
		emit serverDetails(server);
		accept();
	}	
}

bool ServerSettingsDialog::checkValid() {
	if (m_displayName->text().isEmpty()) {
		QMessageBox::critical(this,"Knzb",i18n("Server display name must not be empty"));
		return false;
	}
	
	if (m_hostname->text().isEmpty()) {
		QMessageBox::critical(this,"Knzb",i18n("Server host name must not be empty"));
		return false;
	}
	
	bool isInt;
	int port_no = m_portNo->text().toInt(&isInt,10);
	if (!isInt || port_no<1 || port_no>65535) {
		QMessageBox::critical(this,"Knzb",i18n("The TCP/IP port number must be between 1 and 65535"));
		return false;
	}
	if (m_authRequired->isChecked() && (m_username->text().isEmpty() || m_password->text().isEmpty())) {
		QMessageBox::critical(this,"Knzb",i18n("You must enter a username and password if authentication is enabled"));
		return false;
	}
	return true;
}

void ServerSettingsDialog::modifyServerSettings(Server* server) {
	m_displayName->setText(server->displayName());
	m_displayName->setReadOnly(true);

	m_hostname->setText(server->hostName());
	m_portNo->setText(QString::number(server->port()));
	m_timeout->setValue(server->timeout());
	m_maxConnections->setValue(server->maxThreads());
	m_priority->setValue(server->priority());
	m_enabled->setChecked(server->enabled());
	
	m_authRequired->setChecked(server->authRequired());
	m_username->setText(server->username());
	m_password->setText(server->password());
}	

void ServerSettingsDialog::authRequiredToggled(bool required) {
		m_username->setEnabled(required);
		m_password->setEnabled(required);
}

