/***************************************************************************
                          settings.cpp  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qvaluelist.h>

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>

#include "settings.h"
#include "server.h"

Settings::Settings() {
}

Settings::~Settings() {
	if (!serverList.isEmpty()) emptyServerList();
}

void Settings::loadSettings() {
	KConfig *settings = kapp->config();
	
	settings->setGroup("General");
	tempDir = settings->readEntry("Temporary File Path", "NONE");
	destDir = settings->readEntry("Destination File Path", "NONE");

	skipDownloadedFiles = settings->readEntry(
				"Skip files in download directory","yes")=="yes";
	removeCompletedItemsFromQueue = settings->readEntry(
				"Remove completed items from queue","yes")=="yes";
	closeAppWhenComplete = settings->readEntry(
				"Close application when queue is completed", "no")=="yes";	
	dockToSystray = settings->readEntry("Dock to systray","no")=="yes";
	
	treePerNzb = settings->readEntry("Tree per Nzb","yes")=="yes";
	
	if (!serverList.isEmpty()) {
		emptyServerList();
	}
	
	settings->setGroup("Servers");
	QString serverString;
	int serverCount=0;
	
	while (1) {
		serverString = settings->readEntry("Server"+QString::number(serverCount),"NOSERVERFOUND");
		if (serverString == "NOSERVERFOUND") break;
		QStringList serverDetails = QStringList::split(':',serverString);
		if (serverDetails.count()<8 || serverDetails.count()>10) {
			kdDebug()<<"Found invalid server line in config settings - skipping\n";
			serverCount++;
		}
  		Server *server = new Server();
  	  	server->setDisplayName(*serverDetails.at(0));
		server->setHostName(*serverDetails.at(1));
		server->setPriority((*serverDetails.at(2)).toInt());
		server->setMaxThreads((*serverDetails.at(3)).toInt());
		server->setPort((*serverDetails.at(4)).toInt());
		server->setTimeout((*serverDetails.at(5)).toInt());
		server->setEnabled((*serverDetails.at(6))=="yes");
		if ((*serverDetails.at(7)) =="AUTH" && serverDetails.count()==10) {
			server->setAuthRequired(true);
			server->setUsername(*serverDetails.at(8));
			server->setPassword(*serverDetails.at(9));
		}
		else server->setAuthRequired(false);
		serverList.append(server);
		serverCount++;
	}
}

void Settings::saveSettings() {
	KConfig *settings = kapp->config();
	settings->setGroup("General");
	settings->writeEntry("Temporary File Path",tempDir);
	settings->writeEntry("Destination File Path",destDir);
	settings->writeEntry("Skip files in download directory",
			skipDownloadedFiles?"yes":"no");
	settings->writeEntry("Remove completed items from queue",
			removeCompletedItemsFromQueue?"yes":"no");
	settings->writeEntry("Close application when queue is completed",
			closeAppWhenComplete?"yes":"no");
	settings->writeEntry("Dock to systray",
			dockToSystray?"yes":"no");
	
	settings->writeEntry("Tree per Nzb", treePerNzb?"yes":"no");
  	
	//Purge the old server list
	settings->deleteGroup("Servers");
	
	settings->setGroup("Servers");

	int serverCount=0;
	QValueListIterator<Server*> end = serverList.end();
	for (QValueListIterator<Server*> it=serverList.begin(); it!=end; it++) {
//		kdDebug()<<"Saving server "+(*it)->displayName()+"\n";
		QString serverString;
		serverString=(*it)->displayName()+":";
		serverString+=(*it)->hostName()+":";
		serverString+=QString::number((*it)->priority())+":";
		serverString+=QString::number((*it)->maxThreads())+":";
		serverString+=QString::number((*it)->port())+":";
		serverString+=QString::number((*it)->timeout())+":";
		serverString+=(*it)->enabled()?"yes:":"no:";
		serverString+=(*it)->authRequired()?"AUTH:":"NOAUTH:";
		serverString+=(*it)->username()+":";
		serverString+=(*it)->password();
		settings->writeEntry("Server"+QString::number(serverCount),serverString);
		serverCount++;
	}
}

void Settings::emptyServerList() {
	QValueListIterator<Server*> end = serverList.end();
	for (QValueListIterator<Server*> it = serverList.begin(); it!=end; ++it) {
		delete (*it);
	}
	serverList.clear();
}
