/***************************************************************************
                          uudecoder.cpp  -  description
                             -------------------
    begin                : Sat Jul 3 2004
    copyright            : (C) 2004 by David Pye
    email                : dmp@davidmpye.dyndns.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "uudecoder.h"
#include <qfile.h>
#include <kdebug.h>


uuDecoder::uuDecoder(QStringList fileParts, QString outDirectory, QString outFilename):
		Decoder(fileParts, outDirectory, outFilename) {
		
	m_fileParts = fileParts;
	fileIterator = m_fileParts.begin();

	if (outFilename!=QString::null) {
		m_outputFile.setName(outDirectory+"/"+outFilename);
	}
	else {
		QString encodedFilename = this->encodedFilename();
		if (encodedFilename != QString::null) {
			if (encodedFilename.contains("/")) {
				kdDebug()<<"uuDecoder: Detected / in filename - security risk. Aborting\n";
			}
			else m_outputFile.setName(outDirectory+"/"+encodedFilename);
		}
	}
	m_decodingComplete=false;
}

uuDecoder::~uuDecoder(){
	//An explicit close is almost certainly unnecessary, but to be safe.
	m_outputFile.close();
}	

bool uuDecoder::isDecodable() {
	bool retval=false;
	//The first part alone contains the output filename
	QFile partFile(*(m_fileParts.begin()));
	if (partFile.open(IO_ReadOnly)) {
		//Find the begin tag
		QString line;
		while (partFile.readLine(line,1024)!=-1) {
			//Remove leading and trailing whitespace - we're looking for the begin line
			line = line.stripWhiteSpace();
			if (line.left(5)=="begin") {
				retval=true;
			}
		}
	}
	return retval;
}

QString uuDecoder::encodedFilename() {
	QString filename=QString::null;
	//The first part alone contains the output filename
	QFile partFile(*(m_fileParts.begin()));
	if (partFile.open(IO_ReadOnly)) {
		//Find the begin tag
		QString line;
		while (partFile.readLine(line,1024)!=-1) {
			//Remove leading and trailing whitespace - we're looking for the begin line
			line = line.stripWhiteSpace();
			if (line.left(5)=="begin") {
				//We want everything after "begin 644 "
				filename = line.mid(10);
//				kdDebug()<<"uuDecoder: Filename is "<<filename<<"\n";
				break;
			}
		}
		partFile.close();
	}
	else {
		kdDebug()<<"Unable to open file "<<partFile.name()<<"\n";
	}
	return filename;
}

int uuDecoder::decodeNextPart() {
	int retval = ERR_OK;

	if (m_decodingComplete) return ERR_OK;

	if (!m_outputFile.isOpen()) {
		//If the output file is not open, open it now. 
		//It remains open until the object is destroyed.
		if (!m_outputFile.open(IO_ReadWrite)) {
			kdDebug()<<"Unable to open output file "<<m_outputFile.name()<<" - aborting\n";
			return ERR_WRITE;
		}
	}

	QFile partFile(*fileIterator);
	
	if (!partFile.open(IO_ReadOnly)) {
		kdDebug()<<"uuDecoder: Unable to open file part "+ partFile.name()+"\n";
		retval = ERR_MISSING_PART;
	}
	else {
		unsigned char buf[4096];
		unsigned char *ptr = buf;
		
		QString line;
		unsigned int lineLen;
		while ((lineLen = partFile.readLine(line,1024))!=-1) {
			
			if (line.left(5)=="begin") {
				//We don't want this line - read another
				if (partFile.readLine(line,1024)==-1) break;
			}
	
			//If we get the end line, we don't 'decode' it.
			if (line.left(3)=="end") break;
			//Strip the CRLF from the end of the line	
			line.remove(lineLen-2,2);
			const char *ascii_rep = line.ascii();
			unsigned int output_bytes = ascii_rep[0]-' ';
			//kdDebug()<<"The output bytes count is "<<output_bytes<<"\n";
			//kdDebug()<<"The input bytes count is "<<line_length-1<<"\n";
			for (unsigned int i=1; i<lineLen; i+=4) {
				//Blocks of four characters.
				char data[4];
				memcpy(data, &ascii_rep[i],4);
				for (int j=0; j<4; j++) {
					data[j] = (data[j]-' ')&0x3F;
				}
				*ptr++ = ((data[0]<<2) | ((data[1]>>4)&0x03));
				if (output_bytes>1) {
					*ptr++ = ((data[1]&0x0F)<<4) | ((data[2]>>2) &0x0F);
					if (output_bytes>2) {
						*ptr++ = ((data[2]&0x03)<<6) | ((data[3]) &0x3F);
					}
				}
				output_bytes-=3;
	
				if ((ptr - buf) >= 4092) {
					if (m_outputFile.writeBlock((const char *)buf, ptr - buf)!= ptr-buf) {
				   		kdDebug()<<"Write error when flushing 4092 bytes\n";
			      			retval = ERR_WRITE;
					}
			             	ptr=buf;
				}
			}
			
			if ((m_outputFile.writeBlock((const char *)buf, ptr-buf)) != ptr-buf) {
				kdDebug()<<"Write error when flushing " << ptr-buf <<" bytes\n";
				retval = ERR_WRITE;
			}
		}
		partFile.close();
	}
	fileIterator++;
	if (fileIterator==m_fileParts.end()) {
		m_decodingComplete=true;
	}
	return retval;
}
	
bool uuDecoder::decodingComplete() {
	return m_decodingComplete;
}
